#ifndef CLIENTCORE_H
#define CLIENTCORE_H

#include "ClientFile.h"
#include <InfoClient.hpp>
#include <InfoCommunicate.hpp>
#include <InfoDirFile.hpp>
#include <array>
#include <cstdint>
#include <functional>
#include <memory>
#include <thread>
#include <vector>
#include <wx/socket.h>

#include <Communicate.h>
#include <Util.h>

class ClientCore : public wxEvtHandler
{
public:
    ClientCore();
    ~ClientCore();

public:
    bool Connect(const wxString& host, uint16_t port);
    void Disconnect();

public:
    wxString GetErr() const;
    wxString GetOwnId() const;
    bool IsOk();
    void SetLogCallback(const std::function<void(const wxString&)>& callback);
    bool ReqOnline();
    bool AskDirectory(const wxString& id, const wxString& path, DirFileInfoVec& dirInfoVec);

public:
    void ReqOnlineCallback(const std::function<void(const InfoClientVec&)>& callback);
    void SetHomeCallback(const std::function<void(const wxString&)>& callback);
    void SetDirFileCallback(const std::function<void(const DirFileInfoVec&)>& callback);

private:
    void UseFrame(FrameBuffer* buf);

private:
    void OnSocketEvent(wxSocketEvent& event);

private:
    void HeartBeat();
    void Recv();

public:
    template <typename T> bool Send(const T& info, FrameBufferType type, const wxString& remoteId = wxEmptyString)
    {
        std::stringstream ss;
        cereal::BinaryOutputArchive archive(ss);
        archive(info);

        auto buf = std::make_shared<FrameBuffer>();
        buf->dataConst = ss.view().data();
        buf->len = ss.str().size();
        buf->dataType = type;
        buf->tid = remoteId.ToStdString();

        return Send(buf.get());
    }
    bool Send(FrameBuffer* buf);

private:
    wxString id_;
    bool thRun_;
    MutBuffer buffer_;
    wxString err_;
    ClientFile cf_;
    std::thread recvThread_;
    std::array<char, GBUFFER_SIZE> buf_;
    wxSocketClient* socket_;

private:
    std::function<void(const DirFileInfoVec&)> dirFileCall_;
    std::function<void(const InfoClientVec&)> onlineCallback_;
    std::function<void(const wxString&)> logCall_;
    std::function<void(const wxString&)> homeCall_;
};

#endif   // CLIENTCORE_H